pub struct ParsingOptions { /* private fields */ }Expand description
Parsing options for the crate::Reader to follow.
For now the only option that can be set is hls_tag_names_to_parse. For convenience, a builder
struct ParsingOptionsBuilder has been provided, to make constructing this struct easier.
Implementations
Sourceimpl ParsingOptions
impl ParsingOptions
Sourcepub fn builder() -> ParsingOptionsBuilder
pub fn builder() -> ParsingOptionsBuilder
Start a builder for constructing the ParsingOptions.
Sourcepub fn hls_tag_names_to_parse(&self) -> &HashSet<TagName>
pub fn hls_tag_names_to_parse(&self) -> &HashSet<TagName>
The tag names that will be parsed by the crate::Reader.
HLS tags that are not included in this list will be parsed as
crate::line::HlsLine::UnknownTag.