ParsedLineSlice

Struct ParsedLineSlice 

pub struct ParsedLineSlice<'a, T>
where T: Debug + PartialEq,
{ pub parsed: T, pub remaining: Option<&'a str>, }
A slice of parsed line data from a HLS playlist.

This struct allows us to parse some way into a playlist, breaking on the new line, and providing the remaining characters after the new line in the Self::remaining field. This is a building block type that is used by the crate::Reader to work through an input playlist with each call to crate::Reader::read_line.

Fields

parsed: T

The parsed data from the slice of line data from the playlist.

remaining: Option<&'a str>

The remaining string slice (after new line characters) from the playlist after parsing. If the parsed line was the last in the input data then the remaining is None.