All error types exposed by the library.
The module offers a collection of many error types coming from various operations.
Structs
- Parse
Float Error - An error found when trying to parse a float from a byte slice (
&[u8]). - Parse
Line Bytes Error - Error in reading a line from
crate::line::parse_bytes(orcrate::line::parse_bytes_with_custom). - Parse
Line StrError - Error in reading a line from
crate::line::parse(orcrate::line::parse_with_custom). - Reader
Bytes Error - Error in reading a line from a
crate::Readerconstructed withcrate::Reader::from_bytes. - Reader
StrError - Error in reading a line from a
crate::Readerconstructed withcrate::Reader::from_str. - Unrecognized
Enumeration Error - An enumerated string provided to a parsed tag was not recognized.
Enums
- Attribute
List Parsing Error - An error found while parsing an attribute list.
- Date
Time Syntax Error - An error experienced while trying to parse
crate::date::DateTime. - Decimal
Resolution Parse Error - An error found when trying to parse a decimal resolution (
<width>x<height>). - Generic
Syntax Error - A generic syntax error that breaks parsing of the line.
- Parse
Attribute Value Error - An error in trying to convert an attribute value into a different data type as defined for the attribute in the HLS specification.
- Parse
Decimal Floating Point With Title Error - An error in trying to convert into a decimal float with a title (used in the
EXTINFtag). - Parse
Decimal Integer Range Error - An error when trying to parse a decimal integer range (
<n>[@<o>]). - Parse
MapByterange Error - An error when trying to parse the
EXT-X-MAP:BYTERANGEattribute. - Parse
Number Error - An error found while trying to parse a number.
- Parse
Playlist Type Error - An error when trying to parse the playlist type
- Parse
TagValue Error - An error found trying to convert a tag value into a different data type needed for the tag.
- Syntax
Error - Error experienced during parsing of a line.
- TagValue
Syntax Error - A syntax error found while trying to parse a tag value.
- Unknown
TagSyntax Error - An error experienced while trying to parse
crate::line::HlsLine::UnknownTag. - Validation
Error - An error experienced while trying to convert into a known tag via
TryFrom<ParsedTag>.