pub enum AttributeListParsingError {
EndOfLineWhileReadingAttributeName,
UnexpectedCharacterInAttributeName,
EmptyAttributeName,
EmptyUnquotedValue,
UnexpectedCharacterInAttributeValue,
UnexpectedCharacterAfterQuoteEnd,
EndOfLineWhileReadingQuotedValue,
InvalidUtf8(Utf8Error),
}Expand description
An error found while parsing an attribute list.
Variants
EndOfLineWhileReadingAttributeName
The line ended while reading an attribute name (before we found =).
UnexpectedCharacterInAttributeName
There was an unexpected character (" or ,) in the attribute name.
EmptyAttributeName
The attribute name was empty (no characters between start and =).
EmptyUnquotedValue
The unquoted attribute value was empty (no characters between start and , or end of line).
UnexpectedCharacterInAttributeValue
Unexpected character in unquoted attribute value (= or " after initial index).
UnexpectedCharacterAfterQuoteEnd
Unexpected character occurring after quote end and before ,.
EndOfLineWhileReadingQuotedValue
The line ended while reading a quoted string value.
InvalidUtf8(Utf8Error)
There was an error when trying to convert to UTF-8.