pub enum GenericSyntaxError {
CarriageReturnWithoutLineFeed,
UnexpectedEndOfLine,
InvalidUtf8(Utf8Error),
}Expand description
A generic syntax error that breaks parsing of the line.
Variants
CarriageReturnWithoutLineFeed
Carriage return (U+000D) was encountered and not followed by line feed (U+000A).
UnexpectedEndOfLine
The line ended unexpectedly (e.g. within a quoted string in an attribute list).
InvalidUtf8(Utf8Error)
Some part of the line could not be decoded as UTF-8.