pub enum ParseAttributeValueError {
UnexpectedQuoted {
attr_name: &'static str,
},
UnexpectedUnquoted {
attr_name: &'static str,
},
DecimalInteger {
attr_name: &'static str,
error: ParseNumberError,
},
DecimalFloatingPoint {
attr_name: &'static str,
error: ParseFloatError,
},
DecimalResolution {
attr_name: &'static str,
error: DecimalResolutionParseError,
},
Utf8 {
attr_name: &'static str,
error: Utf8Error,
},
}Expand description
An error in trying to convert an attribute value into a different data type as defined for the attribute in the HLS specification.
Variants
UnexpectedQuoted
The value is wrapped in quotes when it should not be.
UnexpectedUnquoted
The value is not wrapped in quotes when it should be.
DecimalInteger
An issue found trying to convert into a decimal integer.
Fields

error: ParseNumberErrorThe underlying error.
DecimalFloatingPoint
An issue found trying to convert into a decimal floating point.
Fields

error: ParseFloatErrorThe underlying error.
DecimalResolution
An issue found trying to convert into a decimal resolution.
Fields

error: DecimalResolutionParseErrorThe underlying error.
Utf8
An issue found trying to convert into a UTF-8 string.