pub enum ParseTagValueError {
NotEmpty,
UnexpectedEmpty,
DecimalInteger(ParseNumberError),
DecimalIntegerRange(ParseDecimalIntegerRangeError),
PlaylistType(ParsePlaylistTypeError),
DecimalFloatingPoint(ParseFloatError),
DecimalFloatingPointWithTitle(ParseDecimalFloatingPointWithTitleError),
DateTime(DateTimeSyntaxError),
AttributeList(AttributeListParsingError),
}Expand description
An error found trying to convert a tag value into a different data type needed for the tag.
Variants
NotEmpty
The tag value is not empty (None) but it should be.
UnexpectedEmpty
The tag value is empty (None) when it should not be.
DecimalInteger(ParseNumberError)
An issue found trying to convert into a decimal integer.
DecimalIntegerRange(ParseDecimalIntegerRangeError)
An issue fouud trying to convert into a decimal integer range (<n>[@<o>]).
PlaylistType(ParsePlaylistTypeError)
An issue found trying to convert into a playlist type enum (EVENT or VOD).
DecimalFloatingPoint(ParseFloatError)
An issue found trying to convert into a decimal floating point number.
DecimalFloatingPointWithTitle(ParseDecimalFloatingPointWithTitleError)
An issue found trying to convert into a decimal floating point number with a UTF-8 title.
DateTime(DateTimeSyntaxError)
An issue found trying to convert into a date/time.
AttributeList(AttributeListParsingError)
An issue found trying to convert into an attribute list.