pub enum ValidationError {
UnexpectedTagName,
MissingRequiredAttribute(&'static str),
NotImplemented,
ErrorExtractingTagValue(ParseTagValueError),
ErrorExtractingAttributeListValue(ParseAttributeValueError),
InvalidEnumeratedString,
}Expand description
An error experienced while trying to convert into a known tag via TryFrom<ParsedTag>.
Variants
UnexpectedTagName
The tag name did not match expectations for the tag.
MissingRequiredAttribute(&'static str)
A required attribute was missing (the associated value should be the required attribute name).
NotImplemented
Parsing for this tag is not implemented.
ErrorExtractingTagValue(ParseTagValueError)
The expected value of the tag could not be obtained.
ErrorExtractingAttributeListValue(ParseAttributeValueError)
An attribute value within an attribute list could not be parsed.
InvalidEnumeratedString
The enumerated string extracted from crate::tag::UnquotedAttributeValue was not a known
value.