pub struct ReaderStrError<'a> {
pub errored_line: &'a str,
pub error: SyntaxError,
}Expand description
Error in reading a line from a crate::Reader constructed with crate::Reader::from_str.
Fields
errored_line: &'a strThe original line that caused the error.
The Reader exposes this to the user so that it can continue to the next line when
crate::Reader::read_line is called again.
error: SyntaxErrorThe underlying error that was experienced.