AttributeValue

Enum AttributeValue 

pub enum AttributeValue<'a> {
    Unquoted(UnquotedAttributeValue<'a>),
    Quoted(&'a str),
}
An attribute value within an attribute list.

Values may be quoted or unquoted. In the case that they are unquoted they may be converted into several other data types. This is done via use of convenience methods on UnquotedAttributeValue.

Variants



Unquoted(UnquotedAttributeValue<'a>)

An unquoted value (e.g. TYPE=AUDIO, BANDWIDTH=10000000, SCORE=1.5, RESOLUTION=1920x1080, SCTE35-OUT=0xABCD, etc.).



Quoted(&'a str)

A quoted value (e.g. CODECS="avc1.64002a,mp4a.40.2").

Implementations

Source

impl<'a> AttributeValue<'a>

pub fn unquoted(&self) -> Option<UnquotedAttributeValue<'a>>

A convenience method to get the value of the Unquoted case.

This can be useful when chaining on optional values. For example:

fn get_bandwidth(list: &HashMap<&str, AttributeValue>) -> Option<u64> {
    list
        .get("BANDWIDTH")
        .and_then(AttributeValue::unquoted)
        .and_then(|v| v.try_as_decimal_integer().ok())
}
pub fn quoted(&self) -> Option<&'a str>

A convenience method to get the value of the Quoted case.

This can be useful when chaining on optional values. For example:

fn get_codecs<'a>(list: &HashMap<&'a str, AttributeValue<'a>>) -> Option<&'a str> {
    list
        .get("CODECS")
        .and_then(AttributeValue::quoted)
}