InterstitialExtensionAttributesMut

Struct InterstitialExtensionAttributesMut 

pub struct InterstitialExtensionAttributesMut<'a, 'b> { /* private fields */ }
Corresponds to the attributes defined for HLS Interstitials in [Appnedix D].

This provides mutable access to the properties. Setting or unsetting values here will set/unset them on the Daterange from which this was derived from.

Implementations

Source

impl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>

pub fn attrs(&'b self) -> InterstitialExtensionAttributes<'a, 'b>

Provides access to the existing values of the interstitial attributes.

Source

impl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>

pub fn set_asset_uri(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-ASSET-URI attribute.

pub fn unset_asset_uri(&mut self)

Unsets the X-ASSET-URI attribute.

pub fn set_asset_list(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-ASSET-LIST attribute.

pub fn unset_asset_list(&mut self)

Unsets the X-ASSET-LIST attribute.

pub fn set_resume_offset(&mut self, value: f64)

Sets the X-RESUME-OFFSET attribute.

pub fn unset_resume_offset(&mut self)

Unsets the X-RESUME-OFFSET attribute.

pub fn set_playout_limit(&mut self, value: f64)

Sets the X-PLAYOUT-LIMIT attribute.

pub fn unset_playout_limit(&mut self)

Unsets the X-PLAYOUT-LIMIT attribute.

pub fn set_snap(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-SNAP attribute.

pub fn unset_snap(&mut self)

Unsets the X-SNAP attribute.

pub fn set_restrict(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-RESTRICT attribute.

pub fn unset_restrict(&mut self)

Unsets the X-RESTRICT attribute.

pub fn set_content_may_vary(&mut self, value: bool)

Sets the X-CONTENT-MAY-VARY attribute.

pub fn unset_content_may_vary(&mut self)

Unsets the X-CONTENT-MAY-VARY attribute.

pub fn set_timeline_occupies(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-TIMELINE-OCCUPIES attribute.

pub fn unset_timeline_occupies(&mut self)

Unsets the X-TIMELINE-OCCUPIES attribute.

pub fn set_timeline_style(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-TIMELINE-STYLE attribute.

pub fn unset_timeline_style(&mut self)

Unsets the X-TIMELINE-STYLE attribute.

pub fn set_skip_control_offset(&mut self, value: f64)

Sets the X-SKIP-CONTROL-OFFSET attribute.

pub fn unset_skip_control_offset(&mut self)

Unsets the X-SKIP-CONTROL-OFFSET attribute.

pub fn set_skip_control_duration(&mut self, value: f64)

Sets the X-SKIP-CONTROL-DURATION attribute.

pub fn unset_skip_control_duration(&mut self)

Unsets the X-SKIP-CONTROL-DURATION attribute.

pub fn set_skip_control_label_id(&mut self, value: impl Into<Cow<'a, str>>)

Sets the X-SKIP-CONTROL-LABEL-ID attribute.

pub fn unset_skip_control_label_id(&mut self)

Unsets the X-SKIP-CONTROL-LABEL-ID attribute.