Key

Struct Key 

pub struct Key<'a> { /* private fields */ }
Implementations

Source

impl<'a> Key<'a>

pub fn builder() -> KeyBuilder<'a, KeyMethodNeedsToBeSet>

Starts a builder for producing Self.

For example, we could construct a Key as such:

let key = Key::builder()
    .with_method(Method::SampleAes)
    .with_uri("skd://1234")
    .with_keyformat("com.apple.streamingkeydelivery")
    .with_keyformatversions("1")
    .finish();

Note that the finish method is only callable if the builder has set method. The following fail to compile:

let key = Key::builder().finish();
pub fn method(&self) -> EnumeratedString<'_, Method>

Corresponds to the METHOD attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn uri(&self) -> Option<&str>

Corresponds to the URI attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn iv(&self) -> Option<&str>

Corresponds to the IV attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn keyformat(&self) -> &str

Corresponds to the KEYFORMAT attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn keyformatversions(&self) -> Option<&str>

Corresponds to the KEYFORMATVERSIONS attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn set_method(&mut self, method: impl Into<Cow<'a, str>>)

Sets the METHOD attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn set_uri(&mut self, uri: impl Into<Cow<'a, str>>)

Sets the URI attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn unset_uri(&mut self)

Unsets the URI attribute (sets it to None).

See Self for a link to the HLS documentation for this attribute.

pub fn set_iv(&mut self, iv: impl Into<Cow<'a, str>>)

Sets the IV attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn unset_iv(&mut self)

Unsets the IV attribute (sets it to None).

See Self for a link to the HLS documentation for this attribute.

pub fn set_keyformat(&mut self, keyformat: impl Into<Cow<'a, str>>)

Sets the KEYFORMAT attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn unset_keyformat(&mut self)

Unsets the KEYFORMAT attribute (sets it to None).

See Self for a link to the HLS documentation for this attribute.

pub fn set_keyformatversions( &mut self, keyformatversions: impl Into<Cow<'a, str>>, )

Sets the KEYFORMATVERSIONS attribute.

See Self for a link to the HLS documentation for this attribute.

pub fn unset_keyformatversions(&mut self)

Unsets the KEYFORMATVERSIONS attribute (sets it to None).

See Self for a link to the HLS documentation for this attribute.