pub struct PreloadExtensionAttributesMut<'a, 'b> { /* private fields */ }Expand description
Corresponds to the attributes defined for Preloading HLS Date Range Resources in Appendix F.
This provides mutable access to the properties. Setting or unsetting values here will set/unset
them on the Daterange from which this was derived from.
Implementations
Sourceimpl<'a, 'b> PreloadExtensionAttributesMut<'a, 'b>
impl<'a, 'b> PreloadExtensionAttributesMut<'a, 'b>
Sourcepub fn attrs(&'b self) -> PreloadExtensionAttributes<'b>
pub fn attrs(&'b self) -> PreloadExtensionAttributes<'b>
Provides access to the existing values of the preload attributes.
Sourceimpl<'a, 'b> PreloadExtensionAttributesMut<'a, 'b>
impl<'a, 'b> PreloadExtensionAttributesMut<'a, 'b>
Sourcepub fn set_target_id(&mut self, target_id: impl Into<Cow<'a, str>>)
pub fn set_target_id(&mut self, target_id: impl Into<Cow<'a, str>>)
Sets the X-TARGET-ID attribute.
Sourcepub fn set_target_class(&mut self, target_class: impl Into<Cow<'a, str>>)
pub fn set_target_class(&mut self, target_class: impl Into<Cow<'a, str>>)
Sets the X-TARGET-CLASS attribute.