pub struct CustomTagAccess<'a, Custom>where
Custom: CustomTag<'a>,{ /* private fields */ }Expand description
Wrapper around a CustomTag implementation for access control.
The wrapper allows the library to selectively decide when it will call the
WritableCustomTag::into_writable_tag method. When there has been no mutable reference borrow
of the custom tag (Self::as_mut) then the Self::into_inner implementation will use the
original parsed byte-slice directly (rather than allocate any new strings to construct a new
line).