pub struct DecimalIntegerRange {
pub length: u64,
pub offset: Option<u64>,
}Expand description
Represents the decimal-integer-range that is found in several places, from tag values to
attribute values, and has structure <n>[@<o>].
For example:
assert_eq!(
DecimalIntegerRange {
length: 1024,
offset: Some(512)
},
DecimalIntegerRange::try_from("1024@512")?
);Fields
length: u64Corresponds to the length component in the value (n in <n>@<o>).
offset: Option<u64>Corresponds to the offset component in the value (o in <n>@<o>).