NoCustomTag

Struct NoCustomTag 

pub struct NoCustomTag;
Implementation of CustomTag for convenience default HlsLine::Custom implementation.

Given that HlsLine takes a generic parameter, if this struct did not exist, then the user would always have to define some custom tag implementation to use the library. This would add unintended complexity. Therefore, this struct comes with the library, and provides the default implementation of CustomTag. This implementation ensures that it is never parsed from source data, because Self::is_known_name always returns false.