Module error

Module error 

All error types exposed by the library.

The module offers a collection of many error types coming from various operations.

Structs

ParseFloatError
An error found when trying to parse a float from a byte slice (&[u8]).
ParseLineBytesError
Error in reading a line from crate::line::parse_bytes (or crate::line::parse_bytes_with_custom).
ParseLineStrError
Error in reading a line from crate::line::parse (or crate::line::parse_with_custom).
ReaderBytesError
Error in reading a line from a crate::Reader constructed with crate::Reader::from_bytes.
ReaderStrError
Error in reading a line from a crate::Reader constructed with crate::Reader::from_str.
UnrecognizedEnumerationError
An enumerated string provided to a parsed tag was not recognized.

Enums

AttributeListParsingError
An error found while parsing an attribute list.
DateTimeSyntaxError
An error experienced while trying to parse crate::date::DateTime.
DecimalResolutionParseError
An error found when trying to parse a decimal resolution (<width>x<height>).
GenericSyntaxError
A generic syntax error that breaks parsing of the line.
ParseAttributeValueError
An error in trying to convert an attribute value into a different data type as defined for the attribute in the HLS specification.
ParseDecimalFloatingPointWithTitleError
An error in trying to convert into a decimal float with a title (used in the EXTINF tag).
ParseDecimalIntegerRangeError
An error when trying to parse a decimal integer range (<n>[@<o>]).
ParseMapByterangeError
An error when trying to parse the EXT-X-MAP:BYTERANGE attribute.
ParseNumberError
An error found while trying to parse a number.
ParsePlaylistTypeError
An error when trying to parse the playlist type
ParseTagValueError
An error found trying to convert a tag value into a different data type needed for the tag.
SyntaxError
Error experienced during parsing of a line.
TagValueSyntaxError
A syntax error found while trying to parse a tag value.
UnknownTagSyntaxError
An error experienced while trying to parse crate::line::HlsLine::UnknownTag.
ValidationError
An error experienced while trying to convert into a known tag via TryFrom<ParsedTag>.