ParseAttributeValueError

Enum ParseAttributeValueError 

pub enum ParseAttributeValueError {
    UnexpectedQuoted {
        attr_name: &'static str,
    },
    UnexpectedUnquoted {
        attr_name: &'static str,
    },
    DecimalInteger {
        attr_name: &'static str,
        error: ParseNumberError,
    },
    DecimalFloatingPoint {
        attr_name: &'static str,
        error: ParseFloatError,
    },
    DecimalResolution {
        attr_name: &'static str,
        error: DecimalResolutionParseError,
    },
    Utf8 {
        attr_name: &'static str,
        error: Utf8Error,
    },
}
An error in trying to convert an attribute value into a different data type as defined for the attribute in the HLS specification.

Variants



UnexpectedQuoted

The value is wrapped in quotes when it should not be.

Fields

attr_name: &'static str

The name of the attribute.



UnexpectedUnquoted

The value is not wrapped in quotes when it should be.

Fields

attr_name: &'static str

The name of the attribute.



DecimalInteger

An issue found trying to convert into a decimal integer.

Fields

attr_name: &'static str

The name of the attribute.

error: ParseNumberError

The underlying error.



DecimalFloatingPoint

An issue found trying to convert into a decimal floating point.

Fields

attr_name: &'static str

The name of the attribute.

error: ParseFloatError

The underlying error.



DecimalResolution

An issue found trying to convert into a decimal resolution.

Fields

attr_name: &'static str

The name of the attribute.

error: DecimalResolutionParseError

The underlying error.



Utf8

An issue found trying to convert into a UTF-8 string.

Fields

attr_name: &'static str

The name of the attribute.

error: Utf8Error

The underlying error.