pub enum TagValueSyntaxError {
InvalidFloatForDecimalFloatingPointValue,
InvalidUtf8(Utf8Error),
InvalidDecimalInteger(ParseNumberError),
UnexpectedEndOfLineWhileReadingAttributeName,
UnexpectedEmptyAttributeValue,
UnexpectedEndOfLineWithinQuotedString,
UnexpectedCharacterAfterQuotedString(u8),
UnexpectedWhitespaceInAttributeValue,
InvalidFloatInAttributeValue,
Generic(GenericSyntaxError),
}Expand description
A syntax error found while trying to parse a tag value.
Variants
InvalidFloatForDecimalFloatingPointValue
Value was determined to be a decimal floating point but the data was not a valid float.
InvalidUtf8(Utf8Error)
Some part of the value could not be decoded as UTF-8.
InvalidDecimalInteger(ParseNumberError)
Value was determined to be a decimal integer but the data was not a valid number.
UnexpectedEndOfLineWhileReadingAttributeName
The line ended while reading an attribute name in an attribute list.
UnexpectedEmptyAttributeValue
No value existed for an associated attribute name in an attribute list.
UnexpectedEndOfLineWithinQuotedString
The line ended while parsing a quoted string in an attribute list.
UnexpectedCharacterAfterQuotedString(u8)
The quoted string ended and was not immediately followed by , or end of line.
UnexpectedWhitespaceInAttributeValue
An attribute value contained whitespace unexpectedly.
InvalidFloatInAttributeValue
A value was determined to be a floating point but the data was not a valid float.
Generic(GenericSyntaxError)
A generic syntax error that breaks parsing of the line.