pub struct InterstitialExtensionAttributesMut<'a, 'b> { /* private fields */ }Expand description
Corresponds to the attributes defined for HLS Interstitials in [Appnedix D].
This provides mutable access to the properties. Setting or unsetting values here will set/unset
them on the Daterange from which this was derived from.
Implementations
Sourceimpl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>
impl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>
Sourcepub fn attrs(&'b self) -> InterstitialExtensionAttributes<'a, 'b>
pub fn attrs(&'b self) -> InterstitialExtensionAttributes<'a, 'b>
Provides access to the existing values of the interstitial attributes.
Sourceimpl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>
impl<'a, 'b> InterstitialExtensionAttributesMut<'a, 'b>
Sourcepub fn set_asset_uri(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_asset_uri(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-ASSET-URI attribute.
Sourcepub fn unset_asset_uri(&mut self)
pub fn unset_asset_uri(&mut self)
Unsets the X-ASSET-URI attribute.
Sourcepub fn set_asset_list(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_asset_list(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-ASSET-LIST attribute.
Sourcepub fn unset_asset_list(&mut self)
pub fn unset_asset_list(&mut self)
Unsets the X-ASSET-LIST attribute.
Sourcepub fn set_resume_offset(&mut self, value: f64)
pub fn set_resume_offset(&mut self, value: f64)
Sets the X-RESUME-OFFSET attribute.
Sourcepub fn unset_resume_offset(&mut self)
pub fn unset_resume_offset(&mut self)
Unsets the X-RESUME-OFFSET attribute.
Sourcepub fn set_playout_limit(&mut self, value: f64)
pub fn set_playout_limit(&mut self, value: f64)
Sets the X-PLAYOUT-LIMIT attribute.
Sourcepub fn unset_playout_limit(&mut self)
pub fn unset_playout_limit(&mut self)
Unsets the X-PLAYOUT-LIMIT attribute.
Sourcepub fn unset_snap(&mut self)
pub fn unset_snap(&mut self)
Unsets the X-SNAP attribute.
Sourcepub fn set_restrict(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_restrict(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-RESTRICT attribute.
Sourcepub fn unset_restrict(&mut self)
pub fn unset_restrict(&mut self)
Unsets the X-RESTRICT attribute.
Sourcepub fn set_content_may_vary(&mut self, value: bool)
pub fn set_content_may_vary(&mut self, value: bool)
Sets the X-CONTENT-MAY-VARY attribute.
Sourcepub fn unset_content_may_vary(&mut self)
pub fn unset_content_may_vary(&mut self)
Unsets the X-CONTENT-MAY-VARY attribute.
Sourcepub fn set_timeline_occupies(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_timeline_occupies(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-TIMELINE-OCCUPIES attribute.
Sourcepub fn unset_timeline_occupies(&mut self)
pub fn unset_timeline_occupies(&mut self)
Unsets the X-TIMELINE-OCCUPIES attribute.
Sourcepub fn set_timeline_style(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_timeline_style(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-TIMELINE-STYLE attribute.
Sourcepub fn unset_timeline_style(&mut self)
pub fn unset_timeline_style(&mut self)
Unsets the X-TIMELINE-STYLE attribute.
Sourcepub fn set_skip_control_offset(&mut self, value: f64)
pub fn set_skip_control_offset(&mut self, value: f64)
Sets the X-SKIP-CONTROL-OFFSET attribute.
Sourcepub fn unset_skip_control_offset(&mut self)
pub fn unset_skip_control_offset(&mut self)
Unsets the X-SKIP-CONTROL-OFFSET attribute.
Sourcepub fn set_skip_control_duration(&mut self, value: f64)
pub fn set_skip_control_duration(&mut self, value: f64)
Sets the X-SKIP-CONTROL-DURATION attribute.
Sourcepub fn unset_skip_control_duration(&mut self)
pub fn unset_skip_control_duration(&mut self)
Unsets the X-SKIP-CONTROL-DURATION attribute.
Sourcepub fn set_skip_control_label_id(&mut self, value: impl Into<Cow<'a, str>>)
pub fn set_skip_control_label_id(&mut self, value: impl Into<Cow<'a, str>>)
Sets the X-SKIP-CONTROL-LABEL-ID attribute.
Sourcepub fn unset_skip_control_label_id(&mut self)
pub fn unset_skip_control_label_id(&mut self)
Unsets the X-SKIP-CONTROL-LABEL-ID attribute.