pub struct SessionKeyBuilder<'a, MethodStatus, UriStatus> { /* private fields */ }Expand description
A builder for convenience in constructing a SessionKey.
Builder pattern inspired by Sguaba
Implementations
Sourceimpl<'a> SessionKeyBuilder<'a, SessionKeyMethodHasBeenSet, SessionKeyUriHasBeenSet>
impl<'a> SessionKeyBuilder<'a, SessionKeyMethodHasBeenSet, SessionKeyUriHasBeenSet>
Sourcepub fn finish(self) -> SessionKey<'a>
pub fn finish(self) -> SessionKey<'a>
Finish building and construct the SessionKey.
Sourceimpl<'a, MethodStatus, UriStatus> SessionKeyBuilder<'a, MethodStatus, UriStatus>
impl<'a, MethodStatus, UriStatus> SessionKeyBuilder<'a, MethodStatus, UriStatus>
Sourcepub fn with_method(
self,
method: impl Into<Cow<'a, str>>,
) -> SessionKeyBuilder<'a, SessionKeyMethodHasBeenSet, UriStatus>
pub fn with_method( self, method: impl Into<Cow<'a, str>>, ) -> SessionKeyBuilder<'a, SessionKeyMethodHasBeenSet, UriStatus>
Add the provided method to the attributes built into SessionKey.
Sourcepub fn with_uri(
self,
uri: impl Into<Cow<'a, str>>,
) -> SessionKeyBuilder<'a, MethodStatus, SessionKeyUriHasBeenSet>
pub fn with_uri( self, uri: impl Into<Cow<'a, str>>, ) -> SessionKeyBuilder<'a, MethodStatus, SessionKeyUriHasBeenSet>
Add the provided uri to the attributes built into SessionKey.
Sourcepub fn with_iv(self, iv: impl Into<Cow<'a, str>>) -> Self
pub fn with_iv(self, iv: impl Into<Cow<'a, str>>) -> Self
Add the provided iv to the attributes built into SessionKey.
Sourcepub fn with_keyformat(self, keyformat: impl Into<Cow<'a, str>>) -> Self
pub fn with_keyformat(self, keyformat: impl Into<Cow<'a, str>>) -> Self
Add the provided keyformat to the attributes built into SessionKey.
Sourcepub fn with_keyformatversions(
self,
keyformatversions: impl Into<Cow<'a, str>>,
) -> Self
pub fn with_keyformatversions( self, keyformatversions: impl Into<Cow<'a, str>>, ) -> Self
Add the provided keyformatversions to the attributes built into SessionKey.