pub struct TagInner<'a> { /* private fields */ }Expand description
The inner data of a parsed tag.
This struct is primarily useful for the crate::Writer, but can be used outside of writing,
if the user needs to have custom access on the byte-slice content of the tag. The slice the
inner data holds may come from a data source provided during parsing, or may be an owned
Vec<u8> if the tag was mutated or constructed using a builder method for the tag. When the
inner data is a byte slice of parsed data, it may be a slice of the rest of the playlist from
where the tag was found; however, the Self::value method ensures that only the relevant
bytes for this line are provided.
Implementations